import { UserAPI, UserMessageType } from 'wetube-common';
import { WSController, WSEndpointHandler, WSEndpointBindings } from "../websocket";
import { UserService } from "../services";
import { ErrorResponse } from "../contract";
import { handshakeMiddleware } from '../middleware';

export class UserController extends WSController<UserAPI> {
    private userService: UserService;

    constructor(userService: UserService) {
        super();
        this.userService = userService;
    }

    public override register(): WSEndpointBindings<UserAPI> {
        return {
            [UserMessageType.CREATE]: {
                handler: this.createUser,
                middleware: [handshakeMiddleware]
            },
        };
    }

    public createUser: WSEndpointHandler<UserAPI[UserMessageType.CREATE]> = async () => {
        try {
            const newUser = this.userService.createUser();
            return {
                id: newUser.id,
                token: newUser.token
            };
        } catch {
            return ErrorResponse.ServerError;
        }
    };

}