import { WSEndpointMiddleware } from '../websocket';
import { UserService } from '../services';
import { AuthData, ErrorResponse } from '../contract';

export const getAuthMiddleware = (userService: UserService): WSEndpointMiddleware<AuthData> => {
    return (req, connection, next) => {
        const { auth } = req.body;

        if (auth) {
            connection.authToken = auth;
        }

        if (!connection.authToken) {
            next(ErrorResponse.UnauthorizedRequest);
            return;
        }

        try {
            const decoded = userService.validateUser(connection.authToken);
            req.user = decoded;
            next();
        } catch (_) {
            next(ErrorResponse.UnauthorizedRequest);
        }
    };
};