import { AnyObj, WSPacketBody, WSPacketError } from 'wetube-common';
import { WSClientConnection, } from './WSClientConnection';

export type WSEndpoint<Req extends AnyObj = AnyObj, Res extends AnyObj = AnyObj> = {
    request: Req, response: Res,
};
export type WSAnyEndpoints = Record<string, WSEndpoint>;

export type WSEndpointRequest<T extends AnyObj = AnyObj, E extends AnyObj = AnyObj> =
    { [K in keyof E]: E[K] } & { body: WSPacketBody<T> };
export type WSEndpointResponse<T extends AnyObj = AnyObj> =
    Promise<T | WSPacketError | null>
export type WSEndpointHandler<
    R extends WSAnyEndpoints[keyof WSAnyEndpoints],
    ReqExt extends AnyObj = AnyObj
> = (req: WSEndpointRequest<R['request'], ReqExt>, connection: WSClientConnection) => WSEndpointResponse<R['response']>;

// eslint-disable-next-line @typescript-eslint/no-explicit-any
export type WSEndpointMiddleware<ReqExt = any> =
    (req: WSEndpointRequest & ReqExt, connection: WSClientConnection, next: (error?: WSPacketError) => void) => void;

export type WSEndpointBindObj<
    R extends WSAnyEndpoints = WSAnyEndpoints,
    K extends keyof R = keyof R
> = {
    middleware?: WSEndpointMiddleware[],
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    handler: WSEndpointHandler<R[K], any>
}
export type WSEndpointBindings<R extends WSAnyEndpoints> = {
    [K in keyof R]: WSEndpointBindObj<R, K>
}

export abstract class WSController<Endpoints extends WSAnyEndpoints = WSAnyEndpoints> {
    public abstract register(): WSEndpointBindings<Endpoints>;

    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    public onConnection = (connection: WSClientConnection) => { /** noop */ };

    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    public onDisconnect = (connection: WSClientConnection) => { /** noop */ };

    protected get noAck(): null {
        return null;
    }

    protected sendTo<K extends keyof Endpoints>(connection: WSClientConnection, type: K, data: Endpoints[K]['response'] | WSPacketError): void {
        connection.send(type, data);
    }

}