import React from 'react';
import { styled } from '@mui/material/styles';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import Typography from '@mui/material/Typography';
import { Playlist } from './components/Playlist';
import { LobbyList } from './components/LobbyList';
import { TabsPanel } from './components/TabsPanel';
import { Player } from './components/Player';
import { VideoSearch } from './components/VideoSearch';
import { useMediaQuery, useTheme } from '@mui/material';
import { useJoinLobby } from './stores';

const StyledContainer = styled(Box)({
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
    width: '100%',
    height: '100vh',
    margin: '0 auto',
    overflow: 'hidden'
});

const TopBarContainer = styled('div')({
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'space-between',
    width: '100%',
});

const Logo = styled(Typography)({
    margin: 10,
    fontWeight: 'bold',
});

const LeaveButton = styled(Button)({
    margin: 10
});

const StyledVideoSearch = styled(VideoSearch)({
    margin: 10,
}) as typeof VideoSearch;

const StyledBox = styled(Box)(({ theme }) => ({
    flex: 1,
    display: 'flex',
    justifyContent: 'center',
    width: '100%',
    [`${theme.breakpoints.down('md')} and (orientation: portrait)`]: {
        flexDirection: 'column',
        justifyContent: 'flex-start',
    },
}));

const StyledPlayer = styled(Player)(({ theme }) => ({
    flex: 1,
    width: '100%',
    height: '56.26vw',
    maxHeight: 'calc(100vh - 169px)',
    maxWidth: 1280,
    padding: '0 10px',
    [`${theme.breakpoints.down('md')} and (orientation: portrait)`]: {
        flex: 0
    },
})) as typeof Player;

const StyledTabsPanel = styled(TabsPanel)(({ theme }) => ({
    position: 'relative',
    flex: 1,
    maxWidth: 420,
    marginLeft: 10,
    marginRight: 10,
    width: '100%',
    minHeight: 210,
    maxHeight: 600,
    [`${theme.breakpoints.down('md')} and (orientation: portrait)`]: {
        maxWidth: 'unset',
        margin: 0,
        marginTop: 75
    },
})) as typeof TabsPanel;

const StyledPlaylist = styled(Playlist)(() => ({
    position: 'absolute',
    left: 0,
    right: 0,
    bottom: 0,
    top: 48,
})) as typeof Playlist;

const StyledLobbyList = styled(LobbyList)(() => ({
    position: 'absolute',
    left: 0,
    right: 0,
    bottom: 0,
    top: 48,
})) as typeof LobbyList;

export const JoinLobby = () => {
    const theme = useTheme();
    const matches = useMediaQuery(`${theme.breakpoints.up('md')}`);
    const { playlist, player, currentVideo, lobbyUsers, leaveLobby } = useJoinLobby();

    return (
        <StyledContainer>
            <TopBarContainer>
                {matches ? <Logo
                    variant="h5"
                    component="div"
                >
                    {'WeTube'}
                </Logo> : null}
                <StyledVideoSearch
                    onSuggestionPick={playlist.enqueue}
                />
                <LeaveButton
                    onClick={leaveLobby}
                    variant='outlined'
                >
                    {'Leave'}
                </LeaveButton>
            </TopBarContainer>
            <StyledBox>
                <StyledPlayer
                    player={player}
                    videoId={currentVideo || ''}
                    onNextVideo={playlist.dequeue}
                />
                <StyledTabsPanel tabs={['Playlist', 'Lobby']}>
                    <StyledPlaylist
                        items={playlist.items}
                        onItemSelect={playlist.moveUp}
                        onItemDelete={playlist.remove}
                    />
                    <StyledLobbyList items={lobbyUsers} />
                </StyledTabsPanel>
            </StyledBox>
        </StyledContainer>
    );
};