import React from 'react';
import { styled } from '@mui/material/styles';
import Box from '@mui/material/Box';
import YouTube, { YouTubeProps } from 'react-youtube';

import { PlayerControls } from './PlayerControls';
import { usePlayer } from '../stores/usePlayer';

const StyledInternalContainer = styled(Box)({
    maxWidth: '100%',
    maxHeight: '100%',
    aspectRatio: '16 / 9',
    margin: '0 auto',
});

const YouTubePlayer = React.forwardRef((props: YouTubeProps, ref: React.ForwardedRef<YouTube>) => {
    return <YouTube
        {...props}
        ref={ref}
        iframeClassName={props.className}
        className={undefined}
    />;
});

const StyledYouTubePlayer = styled(YouTubePlayer)({
    width: '100%',
    height: '100%',
}) as typeof YouTube;


interface PlayerProps {
    className?: string,
    videoId: string,
    player: ReturnType<typeof usePlayer>
    onNextVideo: () => void,
}
export const Player = (props: PlayerProps) => {
    const {
        player,
        videoId, className,
        onNextVideo,
    } = props;
    const {
        duration, elapsed,
        playing, volume,
        playerRef,
        onPlayerPlay,
        onPlayerPause,
        onPlayerEnd,
        onPlayerStateChange,
        onControlsPlay,
        onControlsPause,
        onControlsSeek,
        onControlsVolumeChange,
    } = player;

    return (
        <Box className={className}>
            <StyledInternalContainer>
                <StyledYouTubePlayer
                    videoId={videoId}
                    id={'wetube-player'}
                    ref={playerRef}
                    style={{
                        width: '100%',
                        height: '100%'
                    }}
                    loading={'eager'}
                    title={''}
                    opts={{
                        playerVars: {
                            controls: 0,
                            autoplay: 1,
                            mute: videoId ? 0 : 1
                        }
                    }}
                    onPlay={onPlayerPlay}
                    onPause={onPlayerPause}
                    onEnd={() => {
                        onPlayerEnd();
                        onNextVideo();
                    }}
                    onStateChange={onPlayerStateChange}
                />
                <PlayerControls
                    duration={duration}
                    elapsed={elapsed}
                    playing={playing}
                    volume={volume}
                    onPlay={onControlsPlay}
                    onPause={onControlsPause}
                    onSeek={onControlsSeek}
                    onVolumeChange={onControlsVolumeChange}
                />
            </StyledInternalContainer>
        </Box>
    );
};