import React from 'react';
import { styled } from '@mui/material/styles';
import List from '@mui/material/List';
import ListItem from '@mui/material/ListItem';
import Collapse from '@mui/material/Collapse';
import { TransitionGroup } from 'react-transition-group';
import { DeletableVideoItem } from './VideoItem';

import { MediaMetadata } from '../../../services/media-metadata';

const StyledList = styled(List)({
    overflowY: 'auto',
});

const StyledDeletableVideoItem = styled(DeletableVideoItem)({
    width: '100%'
});

interface PlaylistProps {
    className?: string;
    items: MediaMetadata[];
    onItemSelect: (metadata: MediaMetadata) => void
    onItemDelete: (metadata: MediaMetadata) => void
}
export const Playlist = (props: PlaylistProps) => {
    const {
        className, items,
        onItemSelect, onItemDelete,
    } = props;

    return (
        <StyledList className={className}>
            <TransitionGroup>
                {items.map((item, i) => (
                    <Collapse key={i}>
                        <ListItem>
                            <StyledDeletableVideoItem
                                metadata={item}
                                onClick={onItemSelect}
                                onDelete={onItemDelete}
                            />
                        </ListItem>
                    </Collapse>
                ))}
            </TransitionGroup>
        </StyledList>
    );
};