import React from 'react';
import { styled } from '@mui/material/styles';
import Box from '@mui/material/Box';
import Tabs from '@mui/material/Tabs';
import Tab from '@mui/material/Tab';

const TAB_ID = 'main-tab';
const TABPANEL_ID = 'main-tabpanel';
const TABPANEL_LABEL = 'main tabpanel';

const StyledTab = styled(Tab)({
    flex: 1,
});

interface TabWrapperProps {
    children?: React.ReactNode;
    index: number;
    value: number;
}
const TabWrapper = (props: TabWrapperProps) => {
    const { children, value, index } = props;

    return (
        <div
            role={'tabpanel'}
            hidden={value !== index}
            id={`${TABPANEL_ID}-${index}`}
            aria-labelledby={`${TAB_ID}-${index}`}
        >
            {value === index ? children : null}
        </div>
    );
};

interface TabPanelProps {
    children?: React.ReactNode;
    className?: string;
    tabs: string[]
}
export const TabsPanel = (props: TabPanelProps) => {
    const { children, className, tabs } = props;

    const [value, setValue] = React.useState(0);
    const handleChange = (_: React.SyntheticEvent, newValue: number) => {
        setValue(newValue);
    };

    return (
        <Box className={className}>
            <Box>
                <Tabs value={value} onChange={handleChange} aria-label={TABPANEL_LABEL}>
                    {tabs.map((tab, index) => (
                        <StyledTab label={tab} key={`${TAB_ID}-${index}`} id={`${TAB_ID}-${index}`} aria-labelledby={`${TABPANEL_ID}-${index}`} />
                    ))}
                </Tabs>
                {React.Children.map(children, (child, ind) => (
                    <TabWrapper value={value} index={ind}>
                        {child}
                    </TabWrapper>
                ))}
            </Box>
        </Box>
    );
};