import { useState, useCallback } from 'react';
import { MediaMetadata } from '../../../services/media-metadata';
import { useAppApi } from '../../../context';

export const useMediaPlaylist = () => {
    const api = useAppApi();
    const [playlist, setPlaylist] = useState<MediaMetadata[]>([]);

    const enqueue = useCallback((media: MediaMetadata): void => {
        setPlaylist((prev) => {
            const newPlaylist = [...prev, media];
            api.syncMediaPlaylist(newPlaylist);

            return newPlaylist;
        });
    }, [playlist]);

    const dequeue = useCallback((): MediaMetadata | null => {
        const result = playlist.shift();
        if (!result) {
            return null;
        }

        const newPlaylist = [...playlist];
        setPlaylist(newPlaylist);
        api.syncMediaPlaylist(newPlaylist);

        return result;
    }, [playlist]);

    const moveUp = useCallback((media: MediaMetadata): void => {
        const indexToMove = playlist.findIndex(item => item.id == media.id);
        if ((indexToMove == -1) || (indexToMove == 0)) {
            return;
        }

        setPlaylist((prev) => {
            const item = playlist[indexToMove];
            prev.splice(indexToMove, 1);

            const newPlaylist = [item, ...prev];
            api.syncMediaPlaylist(newPlaylist);

            return newPlaylist;
        });
    }, [playlist]);

    const remove = useCallback((media: MediaMetadata): void => {
        const indexToRemove = playlist.findIndex(item => item.id == media.id);
        if (indexToRemove == -1) {
            return;
        }

        setPlaylist((prev) => {
            prev.splice(indexToRemove, 1);

            const newPlaylist = [...prev];
            api.syncMediaPlaylist(newPlaylist);

            return newPlaylist;
        });
    }, [playlist]);

    const set = useCallback((items: MediaMetadata[]): void => {
        setPlaylist(items);
    }, []);

    return {
        items: playlist,
        enqueue,
        dequeue,
        remove,
        moveUp,
        set
    };
};