import { useState, useCallback, useEffect } from 'react';
import { useAppApi } from '../../../context';
import { MediaMetadata } from '../../../services/media-metadata';
import { debounce } from '../../../utils';

export const useMediaSearch = () => {
    const api = useAppApi();
    const [mediaMetadata, setMediaMetadata] = useState<MediaMetadata | null>(null);
    const [searchQuery, setSearchQuery] = useState('');

    const getMediaMetadata = useCallback(debounce(async (query: string) => {
        const metadata = await api.getMediaMetadata(query);
        setMediaMetadata(metadata);
    }, 300), []);

    useEffect(() => {
        getMediaMetadata(searchQuery);
    }, [searchQuery]);

    return {
        searchQuery,
        mediaMetadata,
        setSearchQuery,
        setMediaMetadata,
    };
};